@@ -1436,8 +1436,9 @@ sub ExponentialIntegral {
 
   # Use MPFR if possible.
   if ($_have_MPFR < 0) {
-    $_have_MPFR = 1;
-    eval { require Math::MPFR; 1; } or do { $_have_MPFR = 0; };
+    $_have_MPFR = 0;
+    $_have_MPFR = 1 if (!defined $ENV{MPU_NO_MPFR} || $ENV{MPU_NO_MPFR} != 1)
+                    && eval { require Math::MPFR; $Math::MPFR::VERSION>=2.03; };
   }
   # Gotcha -- MPFR decided to make negative inputs return NaN.  Grrr.
   if ($_have_MPFR && $x > 0) {
@@ -1545,8 +1546,9 @@ sub LogarithmicIntegral {
 
   # Use MPFR if possible.
   if ($_have_MPFR < 0) {
-    $_have_MPFR = 1;
-    eval { require Math::MPFR; 1; } or do { $_have_MPFR = 0; };
+    $_have_MPFR = 0;
+    $_have_MPFR = 1 if (!defined $ENV{MPU_NO_MPFR} || $ENV{MPU_NO_MPFR} != 1)
+                    && eval { require Math::MPFR; $Math::MPFR::VERSION>=2.03; };
   }
   # Remember MPFR eint doesn't handle negative inputs
   if ($_have_MPFR && $x >= 1) {
@@ -1677,8 +1679,9 @@ sub RiemannZeta {
 
   # Use MPFR if possible.
   if ($_have_MPFR < 0) {
-    $_have_MPFR = 1;
-    eval { require Math::MPFR; 1; } or do { $_have_MPFR = 0; };
+    $_have_MPFR = 0;
+    $_have_MPFR = 1 if (!defined $ENV{MPU_NO_MPFR} || $ENV{MPU_NO_MPFR} != 1)
+                    && eval { require Math::MPFR; $Math::MPFR::VERSION>=2.03; };
   }
   if ($_have_MPFR) {
     my $wantbf = 0;
@@ -1742,8 +1745,9 @@ sub RiemannR {
 
   # Use MPFR if possible.
   if ($_have_MPFR < 0) {
-    $_have_MPFR = 1;
-    eval { require Math::MPFR; 1; } or do { $_have_MPFR = 0; };
+    $_have_MPFR = 0;
+    $_have_MPFR = 1 if (!defined $ENV{MPU_NO_MPFR} || $ENV{MPU_NO_MPFR} != 1)
+                    && eval { require Math::MPFR; $Math::MPFR::VERSION>=2.03; };
   }
   if ($_have_MPFR) {
     my $wantbf = 0;
